package com.tanpu.fund.service.impl;

import com.tanpu.common.utils.DateUtils;
import com.tanpu.fund.entity.generator.FundLatestNav;
import com.tanpu.fund.entity.generator.FundLatestNavExample;
import com.tanpu.fund.entity.generator.FundNav;
import com.tanpu.fund.mapper.generator.FundLatestNavMapper;
import com.tanpu.fund.mapper.generator.custom.FundInfoCustomMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
public class FundLatestNavService {
    @Resource
    private FundLatestNavMapper fundLatestNavMapper;

    @Resource
    private FundInfoCustomMapper fundInfoCustomMapper;

    public List<FundNav> getLatestNavBy(List<String> fundIdList) {
        if (CollectionUtils.isEmpty(fundIdList)) {
            return new ArrayList<>();
        }

        FundLatestNavExample example = new FundLatestNavExample();
        example.createCriteria().andFundIdIn(fundIdList);
        List<FundLatestNav> navList = fundLatestNavMapper.selectByExample(example);
        return navList.stream().map(p -> {
            FundNav target = new FundNav();
            BeanUtils.copyProperties(p, target);
            return target;
        }).collect(Collectors.toList());
    }

    public void sync(){
        FundLatestNavExample example = new FundLatestNavExample();
        example.createCriteria().andFundIdEqualTo("DF17000000110");
        List<FundLatestNav> list = fundLatestNavMapper.selectByExample(example);
        Date startTime = list.get(0).getCreateTime();
        List<FundLatestNav> latestList = fundInfoCustomMapper.getFundLatestNet(startTime);
        List<FundLatestNav> latestListRH = fundInfoCustomMapper.getFundLatestNetInRH(startTime);
        List<FundLatestNav> navList = new ArrayList<>(latestListRH.size() + latestList.size() + 1);
        navList.addAll(latestList);
        navList.addAll(latestListRH);
        navList.sort((p1, p2) -> {
            int dif = p1.getFundId().compareTo(p2.getFundId());
            if (dif != 0) {
                return dif;
            }
            if (p1.getPriceDate() == null) {
                return -1;
            }
            if (p1.getPriceDate() == null) {
                return 1;
            }
            return p1.getPriceDate().compareTo(p1.getPriceDate());
        });
        log.info("私募新净值统计, count: {}, start: {}", navList.size(), DateUtils.format(startTime));
        if (CollectionUtils.isEmpty(navList)) {
            log.info("私募没有新净值数据，结束");
            return;
        }
        int p = 0;
        while (p < navList.size()) {
            List<FundLatestNav> batchList = navList.subList(p, Math.min(navList.size(), p + 200));
            if (CollectionUtils.isEmpty(batchList)) {
                break;
            }
            List<String> idList = batchList.stream().map(FundLatestNav::getFundId).collect(Collectors.toList());
            FundLatestNavExample latestNavExample = new FundLatestNavExample();
            latestNavExample.createCriteria().andFundIdIn(idList);
            Map<String, List<FundLatestNav>> existMap = fundLatestNavMapper.selectByExample(latestNavExample)
                    .stream().collect(Collectors.groupingBy(FundLatestNav::getFundId));
            FundLatestNavExample updateExample = new FundLatestNavExample();
            for (FundLatestNav latestNav : batchList) {
                // 这几个字段不用更新
                latestNav.setUpdateTime(null);
                latestNav.setId(null);
                latestNav.setCreateTime(null);
                List<FundLatestNav> existNavs = existMap.get(latestNav.getFundId());
                if (CollectionUtils.isNotEmpty(existNavs)) {
                    // update
                    FundLatestNav eNav = existNavs.get(0);
                    if (eNav.getPriceDate().compareTo(latestNav.getPriceDate()) <= 0) {
                        updateExample.clear();
                        updateExample.createCriteria().andFundIdEqualTo(latestNav.getFundId());
                        fundLatestNavMapper.updateByExampleSelective(latestNav, updateExample);
                    }
                } else {
                    // insert
                    fundLatestNavMapper.insertSelective(latestNav);
                }
            }
            p += 200;
        }

        Date newStartTime = latestList.stream().map(FundLatestNav::getCreateTime).max(Date::compareTo).get();
        Date newStartTimeRH = latestListRH.stream().map(FundLatestNav::getCreateTime).max(Date::compareTo).get();
        FundLatestNav target = new FundLatestNav();
        // 存储较小的那个，保证两个表里的都能被扫描到.
        target.setCreateTime(newStartTime.before(newStartTimeRH) ? newStartTime : newStartTimeRH);
        int c = fundLatestNavMapper.updateByExampleSelective(target, example);
        log.info("私募新净值更新起始时间，affected: {}, start: {}", c, DateUtils.format(newStartTime));
    }
}
